package meta

import (
	"encoding/json"
	"errors"
	"fmt"
	"io/fs"
	"os"
	"strings"

	"github.com/adrg/xdg"
)

type Meta struct {
	Packages map[string]string `json:"packages"`
}

func Upsert(name, version string) error {
	m, err := Read()
	if err != nil {
		return err
	}
	for n := range m.Packages {
		if strings.EqualFold(n, name) {
			m.Packages[n] = version
			return save(m)
		}
	}
	m.Packages[name] = version
	return save(m)
}

func Read() (Meta, error) {
	m := Meta{
		Packages: make(map[string]string),
	}
	fp, err := metaPath()
	if err != nil {
		return m, fmt.Errorf("could not get meta file: %w", err)
	}
	fi, err := os.Open(fp)
	if err != nil {
		if errors.Is(err, fs.ErrNotExist) {
			return m, json.Unmarshal([]byte(`{}`), &m)
		}
		return m, fmt.Errorf("could not open meta file: %w", err)
	}
	defer fi.Close()

	if err := json.NewDecoder(fi).Decode(&m); err != nil {
		return m, fmt.Errorf("could not decode meta: %w", err)
	}
	return m, nil
}

func Remove(name string) error {
	m, err := Read()
	if err != nil {
		return err
	}
	for n := range m.Packages {
		if strings.EqualFold(n, name) {
			delete(m.Packages, name)
			return save(m)
		}
	}
	return fmt.Errorf("could not find package to remove for %q", name)
}

func metaPath() (string, error) {
	return xdg.DataFile("eget/meta.json")
}

func save(m Meta) error {
	fp, err := metaPath()
	if err != nil {
		return fmt.Errorf("could not get meta path: %w", err)
	}
	fi, err := os.Create(fp)
	if err != nil {
		return fmt.Errorf("could not create meta file: %w", err)
	}
	defer fi.Close()
	return json.NewEncoder(fi).Encode(m)
}
